@@ -12,7 +12,7 @@ This file only deals with non-GUI configuration features
 sip API incompatibility issue in spyder's non-gui modules)
 """
 
-import codecs
+from glob import glob
 import locale
 import os
 import os.path as osp
@@ -25,7 +25,6 @@ import warnings
 
 # Local imports
 from spyder import __version__
-from spyder.py3compat import is_unicode, PY3, to_text_string, is_text_string
 from spyder.utils import encoding
 
 #==============================================================================
@@ -102,7 +101,7 @@ def use_dev_config_dir(use_dev_config_dir=USE_DEV_CONFIG_DIR):
 # Debug helpers
 #==============================================================================
 # This is needed after restarting and using debug_print
-STDOUT = sys.stdout if PY3 else codecs.getwriter('utf-8')(sys.stdout)
+STDOUT = sys.stdout
 STDERR = sys.stderr
 
 
@@ -118,13 +117,10 @@ def debug_print(*message):
     warnings.warn("debug_print is deprecated; use the logging module instead.")
     if get_debug_level():
         ss = STDOUT
-        if PY3:
-            # This is needed after restarting and using debug_print
-            for m in message:
-                ss.buffer.write(str(m).encode('utf-8'))
-            print('', file=ss)
-        else:
-            print(*message, file=ss)
+        # This is needed after restarting and using debug_print
+        for m in message:
+            ss.buffer.write(str(m).encode('utf-8'))
+        print('', file=ss)
 
 
 #==============================================================================
@@ -147,8 +143,7 @@ def get_conf_subfolder():
     #    embed a PY2 interpreter in PY3)
     # 2. We need to save the list of installed modules (for code
     #    completion) separately for each version
-    if PY3:
-        SUBFOLDER = SUBFOLDER + '-py3'
+    SUBFOLDER = SUBFOLDER + '-py3'
 
     # If running a development/beta version, save config in a separate
     # directory to avoid wiping or contaiminating the user's saved stable
@@ -347,16 +342,6 @@ def is_py2exe_or_cx_Freeze():
     return osp.isfile(osp.join(get_module_path('spyder'), osp.pardir))
 
 
-def is_pynsist():
-    """Return True if this is a pynsist installation of Spyder."""
-    base_path = osp.abspath(osp.dirname(__file__))
-    pkgs_path = osp.abspath(
-        osp.join(base_path, '..', '..', '..', 'pkgs'))
-    if os.environ.get('PYTHONPATH') is not None:
-        return pkgs_path in os.environ.get('PYTHONPATH')
-    return False
-
-
 #==============================================================================
 # Translations
 #==============================================================================
@@ -366,20 +351,24 @@ DEFAULT_LANGUAGE = 'en'
 # This needs to be updated every time a new language is added to spyder, and is
 # used by the Preferences configuration to populate the Language QComboBox
 LANGUAGE_CODES = {
-    'en': u'English',
-    'fr': u'Français',
-    'es': u'Español',
-    'hu': u'Magyar',
-    'pt_BR': u'Português',
-    'ru': u'Русский',
-    'zh_CN': u'简体中文',
-    'ja': u'日本語',
-    'de': u'Deutsch',
-    'pl': u'Polski'
+    'en': 'English',
+    'fr': 'Français',
+    'es': 'Español',
+    'hu': 'Magyar',
+    'pt_BR': 'Português',
+    'ru': 'Русский',
+    'zh_CN': '简体中文',
+    'ja': '日本語',
+    'de': 'Deutsch',
+    'pl': 'Polski',
+    'fa': 'Persian',
+    'hr': 'Croatian',
+    'te': 'Telugu',
+    'uk': 'Ukrainian',
 }
 
 # Disabled languages because their translations are outdated or incomplete
-DISABLED_LANGUAGES = ['hu', 'pl']
+DISABLED_LANGUAGES = ['fa', 'hr', 'hu', 'pl', 'te', 'uk', 'ru']
 
 
 def get_available_translations():
@@ -400,14 +389,19 @@ def get_available_translations():
 
     # Check that there is a language code available in case a new translation
     # is added, to ensure LANGUAGE_CODES is updated.
+    retlangs = []
     for lang in langs:
         if lang not in LANGUAGE_CODES:
             if DEV:
                 error = ('Update LANGUAGE_CODES (inside config/base.py) if a '
-                         'new translation has been added to Spyder')
+                         'new translation has been added to Spyder. '
+                         'Currently missing ' + lang)
                 print(error)  # spyder: test-skip
-            return ['en']
-    return langs
+                return ['en']
+        else:
+            retlangs.append(lang)
+
+    return retlangs
 
 
 def get_interface_language():
@@ -417,10 +411,10 @@ def get_interface_language():
     otherwise it will return DEFAULT_LANGUAGE.
 
     Example:
-    1.) Spyder provides ('en',  'de', 'fr', 'es' 'hu' and 'pt_BR'), if the
+    1.) Spyder provides ('en', 'de', 'fr', 'es' 'hu' and 'pt_BR'), if the
     locale is either 'en_US' or 'en' or 'en_UK', this function will return 'en'
 
-    2.) Spyder provides ('en',  'de', 'fr', 'es' 'hu' and 'pt_BR'), if the
+    2.) Spyder provides ('en', 'de', 'fr', 'es' 'hu' and 'pt_BR'), if the
     locale is either 'pt' or 'pt_BR', this function will return 'pt_BR'
     """
 
@@ -490,8 +484,6 @@ def get_translation(modname, dirname=None):
 
     def translate_dumb(x):
         """Dumb function to not use translations."""
-        if not is_unicode(x):
-            return to_text_string(x, "utf-8")
         return x
 
     locale_path = get_module_data_path(dirname, relpath="locale",
@@ -511,21 +503,23 @@ def get_translation(modname, dirname=None):
     else:
         os.environ["LANGUAGE"] = language  # Works on Linux
 
+    if language == "en":
+        return translate_dumb
+
     import gettext
     try:
-        _trans = gettext.translation(modname, locale_path, codeset="utf-8")
-        lgettext = _trans.lgettext
+        _trans = gettext.translation(modname, locale_path)
 
         def translate_gettext(x):
-            if not PY3 and is_unicode(x):
-                x = x.encode("utf-8")
-            y = lgettext(x)
-            if is_text_string(y) and PY3:
-                return y
-            else:
-                return to_text_string(y, "utf-8")
+            return _trans.gettext(x)
         return translate_gettext
-    except Exception:
+    except Exception as exc:
+        # logging module is not yet initialised at this point
+        print(
+            f"Could not load translations for {language} due to: "
+            f"{exc.__class__.__name__} - {exc}",
+            file=sys.stderr
+        )
         return translate_dumb
 
 
@@ -547,61 +541,32 @@ EXCLUDED_NAMES = ['nan', 'inf', 'infty', 'little_endian', 'colorbar_doc',
 
 
 #==============================================================================
-# Mac application utilities
+# Conda-based installer application utilities
 #==============================================================================
-def running_in_mac_app(pyexec=None):
+def is_conda_based_app(pyexec=sys.executable):
     """
-    Check if Python executable is located inside a standalone Mac app.
+    Check if Spyder is running from the conda-based installer by looking for
+    the `spyder-menu.json` file.
 
-    If no executable is provided, the default will check `sys.executable`, i.e.
-    whether Spyder is running from a standalone Mac app.
-
-    This is important for example for the single_instance option and the
-    interpreter status in the statusbar.
+    If a Python executable is provided, checks if it is in a conda-based
+    installer environment or the root environment thereof.
     """
-    if pyexec is None:
-        pyexec = sys.executable
+    real_pyexec = osp.realpath(pyexec)  # pyexec may be symlink
+    if os.name == 'nt':
+        env_path = osp.dirname(real_pyexec)
+    else:
+        env_path = osp.dirname(osp.dirname(real_pyexec))
 
-    bpath = get_mac_app_bundle_path()
-
-    if bpath and pyexec == osp.join(bpath, 'Contents/MacOS/python'):
+    menu_rel_path = '/Menu/conda-based-app'
+    if (
+        osp.exists(env_path + menu_rel_path)
+        or glob(env_path + '/envs/*' + menu_rel_path)
+    ):
         return True
     else:
         return False
 
 
-def get_mac_app_bundle_path():
-    """
-    Return the full path to the macOS app bundle. Otherwise return None.
-
-    EXECUTABLEPATH environment variable only exists if Spyder is a macOS app
-    bundle. In which case it will always end with
-    "/<app name>.app/Conents/MacOS/Spyder".
-    """
-    app_exe_path = os.environ.get('EXECUTABLEPATH', None)
-    if sys.platform == "darwin" and app_exe_path:
-        return osp.dirname(osp.dirname(osp.dirname(osp.abspath(app_exe_path))))
-    else:
-        return None
-
-
-# =============================================================================
-# Micromamba
-# =============================================================================
-def get_spyder_umamba_path():
-    """Return the path to the Micromamba executable bundled with Spyder."""
-    if running_in_mac_app():
-        path = osp.join(osp.dirname(osp.dirname(__file__)),
-                        'bin', 'micromamba')
-    elif is_pynsist():
-        path = osp.abspath(osp.join(osp.dirname(osp.dirname(__file__)),
-                                    'bin', 'micromamba.exe'))
-    else:
-        path = None
-
-    return path
-
-
 #==============================================================================
 # Reset config files
 #==============================================================================
