@@ -14,7 +14,8 @@ import os
 
 # Third party imports
 from qtpy.QtCore import Qt, QByteArray, QSize, QPoint, Slot
-from qtpy.QtWidgets import QApplication, QDesktopWidget
+from qtpy.QtGui import QIcon, QKeySequence
+from qtpy.QtWidgets import QApplication
 
 # Local imports
 from spyder.api.exceptions import SpyderAPIError
@@ -23,6 +24,7 @@ from spyder.api.plugins import (
 from spyder.api.plugin_registration.decorators import (
     on_plugin_available, on_plugin_teardown)
 from spyder.api.plugin_registration.registry import PLUGIN_REGISTRY
+from spyder.api.shortcuts import SpyderShortcutsMixin
 from spyder.api.translations import _
 from spyder.api.utils import get_class_values
 from spyder.plugins.mainmenu.api import ApplicationMenus, ViewMenuSections
@@ -32,7 +34,7 @@ from spyder.plugins.layout.layouts import (DefaultLayouts,
                                            HorizontalSplitLayout,
                                            MatlabLayout, RLayout,
                                            SpyderLayout, VerticalSplitLayout)
-from spyder.plugins.preferences.widgets.container import PreferencesActions
+from spyder.plugins.preferences.api import PreferencesActions
 from spyder.plugins.toolbar.api import (
     ApplicationToolbars, MainToolbarSections)
 from spyder.py3compat import qbytearray_to_str  # FIXME:
@@ -55,15 +57,18 @@ DEFAULT_LAYOUTS = get_class_values(DefaultLayouts)
 # The current versions are:
 #
 # * Spyder 4: Version 0 (it was the default).
-# * Spyder 5.0.0 to 5.0.5: Version 1 (a bump was required due to the new API).
+# * Spyder 5.0.0: Version 1 (a bump was required due to the new API).
 # * Spyder 5.1.0: Version 2 (a bump was required due to the migration of
 #                            Projects to the new API).
-# * Spyder 5.2.0: Version 3 (a bump was required due to the migration of
+# * Spyder 5.2.0: Version 3 (a bump was required due to the migration of the
 #                            IPython Console to the new API)
-WINDOW_STATE_VERSION = 3
+# * Spyder 6.0.0: Version 4 (a bump was required due to the migration of
+#                            Editor to the new API)
+
+WINDOW_STATE_VERSION = 4
 
 
-class Layout(SpyderPluginV2):
+class Layout(SpyderPluginV2, SpyderShortcutsMixin):
     """
     Layout manager plugin.
     """
@@ -80,11 +85,13 @@ class Layout(SpyderPluginV2):
     def get_name():
         return _("Layout")
 
-    def get_description(self):
+    @staticmethod
+    def get_description():
         return _("Layout manager")
 
-    def get_icon(self):
-        return self.create_icon("history")  # FIXME:
+    @classmethod
+    def get_icon(cls):
+        return QIcon()
 
     def on_initialize(self):
         self._last_plugin = None
@@ -193,7 +200,11 @@ class Layout(SpyderPluginV2):
     def before_mainwindow_visible(self):
         # Update layout menu
         self.update_layout_menu_actions()
-        # Setup layout
+
+        # The layout needs to be applied twice: before and after the main
+        # window is visible (see below). This call avoids weird issues when the
+        # window was not maximized in the last session. See:
+        # https://github.com/spyder-ide/spyder/pull/22232#issuecomment-2224142496
         self.setup_layout(default=False)
 
     def on_mainwindow_visible(self):
@@ -202,6 +213,16 @@ class Layout(SpyderPluginV2):
         # that works as expected.
         self.create_plugins_menu()
 
+        # Setup layout when the window is visible.
+        # This **MUST** be done after creating the plugins menu to correctly
+        # restore the layout from the previous session.
+        # Fixes spyder-ide/spyder#17945 and spyder-ide/spyder#21596
+        self.setup_layout(default=False)
+
+        # Correctly display dock tabbars.
+        # This **MUST** be done after setting up the layout.
+        self._apply_docktabbar_style()
+
         # Restore last visible plugins.
         # This **MUST** be done before running on_mainwindow_visible for the
         # other plugins so that the user doesn't experience sudden jumps in the
@@ -241,6 +262,46 @@ class Layout(SpyderPluginV2):
         self.lock_interface_action.setIcon(icon)
         self.lock_interface_action.setText(text)
 
+    def _apply_docktabbar_style(self):
+        """Apply dock tabbar style."""
+        # Apply style by installing the dockwidget tab event filter.
+        plugins = self.get_dockable_plugins()
+        for plugin in plugins:
+            plugin.dockwidget.is_shown = True
+            plugin.dockwidget.install_tab_event_filter()
+
+    def _update_shortcuts_in_plugins_menu(self, show=True):
+        """
+        Show/hide shortcuts for actions in the plugins menu.
+
+        Notes
+        -----
+        Shortcuts in that menu need to disabled when not visible to prevent
+        plugins to be hidden with them.
+        """
+        for plugin in self.get_dockable_plugins():
+            try:
+                # New API
+                action = plugin.toggle_view_action
+            except AttributeError:
+                # Old API
+                action = plugin._toggle_view_action
+
+            if show:
+                section = plugin.CONF_SECTION
+                try:
+                    context = '_'
+                    name = 'switch to {}'.format(section)
+                    shortcut = self.get_shortcut(
+                        name, context, plugin_name=section
+                    )
+                except (cp.NoSectionError, cp.NoOptionError):
+                    shortcut = QKeySequence()
+            else:
+                shortcut = QKeySequence()
+
+            action.setShortcut(shortcut)
+
     # ---- Helper methods
     # -------------------------------------------------------------------------
     def get_last_plugin(self):
@@ -329,29 +390,34 @@ class Layout(SpyderPluginV2):
             self._first_spyder_run = True
             self.setup_default_layouts(DefaultLayouts.SpyderLayout, settings)
 
-            # Now that the initial setup is done, copy the window settings,
-            # except for the hexstate in the quick layouts sections for the
-            # default layouts.
-            # Order and name of the default layouts is found in config.py
-            section = 'quick_layouts'
-            get_func = self.get_conf_default if default else self.get_conf
-            order = get_func('order', section=section)
-
-            # Restore the original defaults if reset layouts is called
+            # Restore the original defaults. This is necessary, for instance,
+            # when bumping WINDOW_STATE_VERSION because layouts saved with a
+            # previous version can't be applied with the new one
             if default:
-                self.set_conf('active', order, section)
-                self.set_conf('order', order, section)
-                self.set_conf('names', order, section)
-                self.set_conf('ui_names', order, section)
+                order = list(self.get_container().spyder_layouts.keys())
+                self.set_conf('order', order)
+                self.set_conf('active', order)
+                self.set_conf('names', order)
 
+                ui_names = [
+                    l.get_name()
+                    for l in self.get_container().spyder_layouts.values()
+                ]
+                self.set_conf('ui_names', ui_names)
+
+                # This will remove custom layouts from the UI
+                self.get_container().update_layout_menu_actions()
+
+            section = 'quick_layouts'
+            order = self.get_conf('order')
             for index, _name, in enumerate(order):
                 prefix = 'layout_{0}/'.format(index)
-                self.save_current_window_settings(prefix, section,
-                                                  none_state=True)
+                self.save_current_window_settings(
+                    prefix, section, none_state=True
+                )
 
             # Store the initial layout as the default in spyder
             prefix = 'layout_default/'
-            section = 'quick_layouts'
             self.save_current_window_settings(prefix, section, none_state=True)
             self._current_quick_layout = DefaultLayouts.SpyderLayout
 
@@ -409,13 +475,16 @@ class Layout(SpyderPluginV2):
         ----------
         index_or_layout_id: int or str
         """
+        # We need to do this first so the new layout is applied as expected.
+        self.unmaximize_dockwidget()
+
         section = 'quick_layouts'
         container = self.get_container()
         try:
             settings = self.load_window_settings(
-                'layout_{}/'.format(index_or_layout_id), section=section)
-            (hexstate, window_size, prefs_dialog_size, pos, is_maximized,
-             is_fullscreen) = settings
+                'layout_{}/'.format(index_or_layout_id), section=section
+            )
+            hexstate, window_size, pos, is_maximized, is_fullscreen = settings
 
             # The defaults layouts will always be regenerated unless there was
             # an overwrite, either by rewriting with same name, or by deleting
@@ -457,6 +526,10 @@ class Layout(SpyderPluginV2):
                 action = plugin._toggle_view_action
             action.setChecked(plugin.dockwidget.isVisible())
 
+        # This is necessary to restore the style for dock tabbars after the
+        # switch
+        self._apply_docktabbar_style()
+
         return index_or_layout_id
 
     def load_window_settings(self, prefix, default=False, section='main'):
@@ -471,8 +544,6 @@ class Layout(SpyderPluginV2):
         """
         get_func = self.get_conf_default if default else self.get_conf
         window_size = get_func(prefix + 'size', section=section)
-        prefs_dialog_size = get_func(
-            prefix + 'prefs_dialog_size', section=section)
 
         if default:
             hexstate = None
@@ -484,20 +555,23 @@ class Layout(SpyderPluginV2):
 
         pos = get_func(prefix + 'position', section=section)
 
+        # We use `virtualGeometry` instead of `geometry` below because it gives
+        # the shape of all connected screens, which is what we need here (
+        # `geometry` only works for the current one).
+        screen_shape = self.main.screen().virtualGeometry()
+        current_width = screen_shape.width()
+        current_height = screen_shape.height()
+
         # It's necessary to verify if the window/position value is valid
         # with the current screen. See spyder-ide/spyder#3748.
         width = pos[0]
         height = pos[1]
-        screen_shape = QApplication.desktop().geometry()
-        current_width = screen_shape.width()
-        current_height = screen_shape.height()
         if current_width < width or current_height < height:
             pos = self.get_conf_default(prefix + 'position', section)
 
         is_maximized = get_func(prefix + 'is_maximized', section=section)
         is_fullscreen = get_func(prefix + 'is_fullscreen', section=section)
-        return (hexstate, window_size, prefs_dialog_size, pos, is_maximized,
-                is_fullscreen)
+        return (hexstate, window_size, pos, is_maximized, is_fullscreen)
 
     def get_window_settings(self):
         """
@@ -515,25 +589,19 @@ class Layout(SpyderPluginV2):
             is_maximized = self.main.isMaximized()
 
         pos = (self.window_position.x(), self.window_position.y())
-        prefs_dialog_size = (self.prefs_dialog_size.width(),
-                             self.prefs_dialog_size.height())
 
         hexstate = qbytearray_to_str(
             self.main.saveState(version=WINDOW_STATE_VERSION)
         )
-        return (hexstate, window_size, prefs_dialog_size, pos, is_maximized,
-                is_fullscreen)
+        return (hexstate, window_size, pos, is_maximized, is_fullscreen)
 
-    def set_window_settings(self, hexstate, window_size, prefs_dialog_size,
-                            pos, is_maximized, is_fullscreen):
+    def set_window_settings(self, hexstate, window_size, pos, is_maximized,
+                            is_fullscreen):
         """
         Set window settings Symetric to the 'get_window_settings' accessor.
         """
         main = self.main
         main.setUpdatesEnabled(False)
-        self.prefs_dialog_size = QSize(prefs_dialog_size[0],
-                                       prefs_dialog_size[1])  # width,height
-        main.set_prefs_size(self.prefs_dialog_size)
         self.window_size = QSize(window_size[0],
                                  window_size[1])  # width, height
         self.window_position = QPoint(pos[0], pos[1])  # x,y
@@ -582,18 +650,12 @@ class Layout(SpyderPluginV2):
         # Fixes spyder-ide/spyder#13882
         win_size = self.main.size()
         pos = self.main.pos()
-        prefs_size = self.prefs_dialog_size
 
         self.set_conf(
             prefix + 'size',
             (win_size.width(), win_size.height()),
             section=section,
         )
-        self.set_conf(
-            prefix + 'prefs_dialog_size',
-            (prefs_size.width(), prefs_size.height()),
-            section=section,
-        )
         self.set_conf(
             prefix + 'is_maximized',
             self.main.isMaximized(),
@@ -664,6 +726,12 @@ class Layout(SpyderPluginV2):
         First call: maximize current dockwidget
         Second call (or restore=True): restore original window layout
         """
+        editor = self.get_plugin(Plugins.Editor, error=False)
+        outline_explorer = self.get_plugin(
+            Plugins.OutlineExplorer,
+            error=False
+        )
+
         if self._state_before_maximizing is None:
             if restore:
                 return
@@ -686,14 +754,14 @@ class Layout(SpyderPluginV2):
                     if plugin.isAncestorOf(focus_widget):
                         self._last_plugin = plugin
 
-            # Only plugins that have a dockwidget are part of widgetlist,
-            # so last_plugin can be None after the above "for" cycle.
-            # For example, this happens if, after Spyder has started, focus
-            # is set to the Working directory toolbar (which doesn't have
-            # a dockwidget) and then you press the Maximize button
+            # This prevents a possible error when the value of _last_plugin
+            # turns out to be None.
             if self._last_plugin is None:
-                # Using the Editor as default plugin to maximize
-                self._last_plugin = self.get_plugin(Plugins.Editor)
+                # Use the Editor as default plugin to maximize
+                if editor is not None:
+                    self._last_plugin = editor
+                else:
+                    return
 
             # Maximize last_plugin
             self._last_plugin.dockwidget.toggleViewAction().setDisabled(True)
@@ -718,13 +786,10 @@ class Layout(SpyderPluginV2):
                 self._last_plugin.show()
                 self._last_plugin._visibility_changed(True)
 
-            if self._last_plugin is self.main.editor:
-                # Automatically show the outline if the editor was maximized:
-                outline_explorer = self.get_plugin(Plugins.OutlineExplorer)
-                self.main.addDockWidget(
-                    Qt.RightDockWidgetArea,
-                    outline_explorer.dockwidget)
-                outline_explorer.dockwidget.show()
+            if self._last_plugin is editor:
+                # Automatically show the outline if the editor was maximized
+                if outline_explorer is not None:
+                    outline_explorer.dock_with_maximized_editor()
         else:
             # Restore original layout (before maximizing current dockwidget)
             try:
@@ -734,6 +799,7 @@ class Layout(SpyderPluginV2):
             except AttributeError:
                 # Old API
                 self._last_plugin.dockwidget.setWidget(self._last_plugin)
+
             self._last_plugin.dockwidget.toggleViewAction().setEnabled(True)
             self.main.setCentralWidget(None)
 
@@ -748,6 +814,11 @@ class Layout(SpyderPluginV2):
                 self._state_before_maximizing, version=WINDOW_STATE_VERSION
             )
             self._state_before_maximizing = None
+
+            if self._last_plugin is editor:
+                if outline_explorer is not None:
+                    outline_explorer.hide_from_maximized_editor()
+
             try:
                 # New API
                 self._last_plugin.get_widget().get_focus_widget().setFocus()
@@ -868,11 +939,7 @@ class Layout(SpyderPluginV2):
                                     | Qt.FramelessWindowHint
                                     | Qt.WindowStaysOnTopHint)
 
-                screen_number = QDesktopWidget().screenNumber(main)
-                if screen_number < 0:
-                    screen_number = 0
-
-                r = QApplication.desktop().screenGeometry(screen_number)
+                r = main.screen().geometry()
                 main.setGeometry(
                     r.left() - 1, r.top() - 1, r.width() + 2, r.height() + 2)
                 main.showNormal()
@@ -889,11 +956,27 @@ class Layout(SpyderPluginV2):
         """
         Populate panes menu with the toggle view action of each base plugin.
         """
-        order = ['editor', 'ipython_console', 'variable_explorer',
-                 'help', 'plots', None, 'explorer', 'outline_explorer',
-                 'project_explorer', 'find_in_files', None, 'historylog',
-                 'profiler', 'breakpoints', 'pylint', None,
-                 'onlinehelp', 'internal_console', None]
+        order = [
+            "editor",
+            "ipython_console",
+            "variable_explorer",
+            "debugger",
+            "help",
+            "plots",
+            None,
+            "explorer",
+            "outline_explorer",
+            "project_explorer",
+            "find_in_files",
+            None,
+            "historylog",
+            "profiler",
+            "pylint",
+            None,
+            "onlinehelp",
+            "internal_console",
+            None,
+        ]
 
         for plugin in self.get_dockable_plugins():
             try:
@@ -927,7 +1010,18 @@ class Layout(SpyderPluginV2):
         actions = order[:]
         for action in actions:
             if type(action) is not str:
-                self.get_container()._plugins_menu.add_action(action)
+                self.plugins_menu.add_action(action)
+
+        # Enable shortcuts when the menu is visible so users can see they are
+        # available. And disable those shortcuts when the menu is hidden
+        # because they allow to hide plugins when pressed twice. See:
+        # https://github.com/spyder-ide/spyder/issues/22189#issuecomment-2248644546
+        self.plugins_menu.aboutToShow.connect(
+            lambda: self._update_shortcuts_in_plugins_menu(show=True)
+        )
+        self.plugins_menu.aboutToHide.connect(
+            lambda: self._update_shortcuts_in_plugins_menu(show=False)
+        )
 
     @property
     def lock_interface_action(self):
@@ -971,8 +1065,12 @@ class Layout(SpyderPluginV2):
 
         # This should only be necessary the first time this method is run
         if not visible_plugins:
-            visible_plugins = [Plugins.IPythonConsole, Plugins.Help,
-                               Plugins.Editor]
+            visible_plugins = [
+                Plugins.IPythonConsole,
+                Plugins.Help,
+                Plugins.VariableExplorer,  # In case Help is not available
+                Plugins.Editor,
+            ]
 
         # Restore visible plugins
         for plugin in visible_plugins:
@@ -1036,7 +1134,14 @@ class Layout(SpyderPluginV2):
             return False
 
         # Get the actual plugins from their names
-        next_to_plugins = [self.get_plugin(p) for p in next_to_plugins]
+        next_to_plugins = [
+            self.get_plugin(p, error=False) for p in next_to_plugins
+        ]
+
+        # Remove not available plugins from next_to_plugins
+        next_to_plugins = [
+            p for p in next_to_plugins if p is not None
+        ]
 
         if plugin.get_conf('first_time', True):
             # This tabifies external and internal plugins that are loaded for
@@ -1122,10 +1227,9 @@ class Layout(SpyderPluginV2):
                 if plugin:
                     self.tabify_plugin(plugin, Plugins.Console)
 
-        # Tabify new external plugins
+        # Tabify any new plugin that was not available in the previous session.
+        # This can include internal plugins that were automatically disabled
+        # in the first session (e.g. due to the lack of WebEngine).
         for plugin in self.get_dockable_plugins():
-            if (
-                plugin.NAME in PLUGIN_REGISTRY.external_plugins
-                and plugin.get_conf('first_time', True)
-            ):
+            if plugin.get_conf('first_time', True):
                 self.tabify_plugin(plugin, Plugins.Console)
