@@ -7,11 +7,8 @@
 # -----------------------------------------------------------------------------
 """Project creation dialog."""
 
-from __future__ import print_function
-
 # Standard library imports
 import errno
-import os
 import os.path as osp
 import sys
 import tempfile
@@ -19,15 +16,16 @@ import tempfile
 # Third party imports
 from qtpy.compat import getexistingdirectory
 from qtpy.QtCore import Qt, Signal
-from qtpy.QtWidgets import (QComboBox, QDialog, QDialogButtonBox, QGridLayout,
-                            QGroupBox, QHBoxLayout, QLabel, QLineEdit,
+from qtpy.QtWidgets import (QDialog, QDialogButtonBox, QGridLayout,
+                            QGroupBox, QHBoxLayout, QLabel, QLayout, QLineEdit,
                             QPushButton, QRadioButton, QVBoxLayout)
 
 # Local imports
+from spyder.api.widgets.comboboxes import SpyderComboBox
+from spyder.api.widgets.dialogs import SpyderDialogButtonBox
 from spyder.config.base import _, get_home_dir
 from spyder.utils.icon_manager import ima
 from spyder.utils.qthelpers import create_toolbutton
-from spyder.py3compat import to_text_string
 
 
 def is_writable(path):
@@ -69,18 +67,19 @@ class ProjectDialog(QDialog):
         self.setWindowFlags(
             self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
 
-        # Variables
-        current_python_version = '.'.join([to_text_string(sys.version_info[0]),
-                                           to_text_string(sys.version_info[1])])
-        python_versions = ['2.7', '3.4', '3.5']
-        if current_python_version not in python_versions:
-            python_versions.append(current_python_version)
-            python_versions = sorted(python_versions)
-
         self.project_name = None
         self.location = get_home_dir()
 
         # Widgets
+        projects_url = "http://docs.spyder-ide.org/current/panes/projects.html"
+        self.description_label = QLabel(
+            _("Select a new or existing directory to create a new Spyder "
+              "project in it. To learn more about projects, take a look at "
+              "our <a href=\"{0}\">documentation</a>.").format(projects_url)
+        )
+        self.description_label.setOpenExternalLinks(True)
+        self.description_label.setWordWrap(True)
+
         self.groupbox = QGroupBox()
         self.radio_new_dir = QRadioButton(_("New directory"))
         self.radio_from_dir = QRadioButton(_("Existing directory"))
@@ -88,29 +87,28 @@ class ProjectDialog(QDialog):
         self.label_project_name = QLabel(_('Project name'))
         self.label_location = QLabel(_('Location'))
         self.label_project_type = QLabel(_('Project type'))
-        self.label_python_version = QLabel(_('Python version'))
 
         self.text_project_name = QLineEdit()
         self.text_location = QLineEdit(get_home_dir())
-        self.combo_project_type = QComboBox()
-        self.combo_python_version = QComboBox()
+        self.combo_project_type = SpyderComboBox()
 
         self.label_information = QLabel("")
+        self.label_information.hide()
 
         self.button_select_location = create_toolbutton(
             self,
             triggered=self.select_location,
             icon=ima.icon('DirOpenIcon'),
-            tip=_("Select directory"))
+            tip=_("Select directory")
+        )
         self.button_cancel = QPushButton(_('Cancel'))
         self.button_create = QPushButton(_('Create'))
 
-        self.bbox = QDialogButtonBox(Qt.Horizontal)
+        self.bbox = SpyderDialogButtonBox(orientation=Qt.Horizontal)
         self.bbox.addButton(self.button_cancel, QDialogButtonBox.ActionRole)
         self.bbox.addButton(self.button_create, QDialogButtonBox.ActionRole)
 
         # Widget setup
-        self.combo_python_version.addItems(python_versions)
         self.radio_new_dir.setChecked(True)
         self.text_location.setEnabled(True)
         self.text_location.setReadOnly(True)
@@ -121,18 +119,14 @@ class ProjectDialog(QDialog):
                             in project_types.items()]:
             self.combo_project_type.addItem(name, id_)
 
-        self.combo_python_version.setCurrentIndex(
-            python_versions.index(current_python_version))
         self.setWindowTitle(_('Create new project'))
-        self.setFixedWidth(500)
-        self.label_python_version.setVisible(False)
-        self.combo_python_version.setVisible(False)
 
         # Layouts
         layout_top = QHBoxLayout()
         layout_top.addWidget(self.radio_new_dir)
+        layout_top.addSpacing(15)
         layout_top.addWidget(self.radio_from_dir)
-        layout_top.addStretch(1)
+        layout_top.addSpacing(200)
         self.groupbox.setLayout(layout_top)
 
         layout_grid = QGridLayout()
@@ -143,17 +137,17 @@ class ProjectDialog(QDialog):
         layout_grid.addWidget(self.button_select_location, 1, 2)
         layout_grid.addWidget(self.label_project_type, 2, 0)
         layout_grid.addWidget(self.combo_project_type, 2, 1, 1, 2)
-        layout_grid.addWidget(self.label_python_version, 3, 0)
-        layout_grid.addWidget(self.combo_python_version, 3, 1, 1, 2)
-        layout_grid.addWidget(self.label_information, 4, 0, 1, 3)
+        layout_grid.addWidget(self.label_information, 3, 0, 1, 3)
 
         layout = QVBoxLayout()
+        layout.addWidget(self.description_label)
+        layout.addSpacing(3)
         layout.addWidget(self.groupbox)
-        layout.addSpacing(10)
+        layout.addSpacing(8)
         layout.addLayout(layout_grid)
-        layout.addStretch()
-        layout.addSpacing(20)
+        layout.addSpacing(8)
         layout.addWidget(self.bbox)
+        layout.setSizeConstraint(QLayout.SizeConstraint.SetFixedSize)
 
         self.setLayout(layout)
 
@@ -174,36 +168,70 @@ class ProjectDialog(QDialog):
             )
         )
 
-        if location:
+        if location and location != '.':
             if is_writable(location):
                 self.location = location
+                self.text_project_name.setText(osp.basename(location))
                 self.update_location()
 
     def update_location(self, text=''):
-        """Update text of location."""
-        self.text_project_name.setEnabled(self.radio_new_dir.isChecked())
+        """Update text of location and validate it."""
+        msg = ''
+        path_validation = False
+        path = self.location
         name = self.text_project_name.text().strip()
 
-        if name and self.radio_new_dir.isChecked():
-            path = osp.join(self.location, name)
-            self.button_create.setDisabled(os.path.isdir(path))
-        elif self.radio_from_dir.isChecked():
-            self.button_create.setEnabled(True)
-            path = self.location
-        else:
-            self.button_create.setEnabled(False)
-            path = self.location
+        # Setup
+        self.text_project_name.setEnabled(self.radio_new_dir.isChecked())
+        self.label_information.setText('')
+        self.label_information.hide()
 
+        if name and self.radio_new_dir.isChecked():
+            # Allow to create projects only on new directories.
+            path = osp.join(self.location, name)
+            path_validation = not osp.isdir(path)
+            if not path_validation:
+                msg = _("This directory already exists!")
+        elif self.radio_from_dir.isChecked():
+            # Allow to create projects in current directories that are not
+            # Spyder projects.
+            path = self.location
+            path_validation = not osp.isdir(osp.join(path, '.spyproject'))
+            if not path_validation:
+                msg = _("This directory is already a Spyder project!")
+
+        # Set path in text_location
         self.text_location.setText(path)
 
-        # Validate name with the method from the currently selected project
+        # Validate project name with the method from the currently selected
+        # project.
         project_type_id = self.combo_project_type.currentData()
         validate_func = self._project_types[project_type_id].validate_name
-        validated, msg = validate_func(path, name)
-        msg = "" if validated else msg
-        self.label_information.setText(msg)
+        project_name_validation, project_msg = validate_func(path, name)
+        if not project_name_validation:
+            if msg:
+                msg = msg + '\n\n' + project_msg
+            else:
+                msg = project_msg
+
+        # Set message
+        if msg:
+            self.label_information.show()
+            self.label_information.setText('\n' + msg)
+
+        # Allow to create project if validation was successful
+        validated = path_validation and project_name_validation
         self.button_create.setEnabled(validated)
 
+        # Set default state of buttons according to validation
+        # Fixes spyder-ide/spyder#16745
+        if validated:
+            self.button_create.setDefault(True)
+            self.button_create.setAutoDefault(True)
+        else:
+            self.button_cancel.setDefault(True)
+            self.button_cancel.setAutoDefault(True)
+
     def create_project(self):
         """Create project."""
         self.project_data = {
@@ -233,7 +261,6 @@ def test():
         def validate_name(path, name):
             return False, "BOOM!"
 
-
     app = qapplication()
     dlg = ProjectDialog(None, {"empty": MockProjectType})
     dlg.show()
